local mod = UpgradeList:NewModule("ItemLocations")

local UPGRADELIST_VERSION = GetAddOnMetadata('UpgradeList', 'Version')

local GeneralItemLocations = {
	"Raid",
	"Dungeon",
	"Crafted",
	"Quest Reward",
	"Reputation Reward",
	"Purchasable",
	"World Drop",
	"Other"
};
	
local InstanceTypes = {
	"Heroic",
	"Normal"
};

local Professions = {
	"Archaeology",
	"Alchemy",
	"Blacksmithing",
	"Enchanting",
	"Engineering",
	"Inscription",
	"Jewelcrafting",
	"Leatherworking",
	"Tailoring"
};

local Reputations = {
	"Guardians of Hyjal",
	"The Earthen Ring",
	"Therazane",
	"Ramkahen",
};

local AllianceReputations = {
	"Wildhammer Clan",
	"Baradin's Wardens",
};

local HordeReputations = {
	"Dragonmaw Clan",
	"Hellscream's Reach",
};

local ReputationLevels = {
	"Exalted",
	"Revered",
	"Honored",
	"Friendly"
};

local Currency = {
	"Gear Token",
	"Valor Points",
	"Justice Points",
	"Conquest Points",
	"Honor Points"
};

local TokenTypes = {
	Tier12 = {
		DDMR = { --Death Knight, Druid, Mage, Rogue
			{ "Crown of the Forlorn Vanquisher", "Nefarian or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Shoulders of the Forlorn Vanquisher", "Cho'gall or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Chest of the Forlorn Vanquisher", "Halfus Wyrmbreaker or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Gauntlets of the Forlorn Vanquisher", "Magmaw or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Leggings of the Forlorn Vanquisher", "Maloriak or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Helm of the Forlorn Vanquisher", "Nefarian", "Normal" },
			{ "Mantle of the Forlorn Vanquisher", "Cho'gall", "Normal" }
		};
		PPW = { --Paladin, Priest, Warlock
			{ "Crown of the Forlorn Conqueror", "Nefarian or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Shoulders of the Forlorn Conqueror", "Cho'gall or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Chest of the Forlorn Conqueror", "Halfus Wyrmbreaker or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Gauntlets of the Forlorn Conqueror", "Magmaw or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Leggings of the Forlorn Conqueror", "Maloriak or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Helm of the Forlorn Conqueror", "Nefarian", "Normal" },
			{ "Mantle of the Forlorn Conqueror", "Cho'gall", "Normal" }
		};
		HSW = { --Hunter, Shaman, Warrior
			{ "Crown of the Forlorn Protector", "Nefarian or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Shoulders of the Forlorn Protector", "Cho'gall or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Chest of the Forlorn Protector", "Halfus Wyrmbreaker or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Gauntlets of the Forlorn Protector", "Magmaw or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Leggings of the Forlorn Protector", "Maloriak or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Helm of the Forlorn Protector", "Nefarian", "Normal" },
			{ "Mantle of the Forlorn Protector", "Cho'gall", "Normal" }
		};
	};
	Tier11 = {
		DDMR = { --Death Knight, Druid, Mage, Rogue
			{ "Crown of the Forlorn Vanquisher", "Nefarian or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Shoulders of the Forlorn Vanquisher", "Cho'gall or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Chest of the Forlorn Vanquisher", "Halfus Wyrmbreaker or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Gauntlets of the Forlorn Vanquisher", "Magmaw or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Leggings of the Forlorn Vanquisher", "Maloriak or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Helm of the Forlorn Vanquisher", "Nefarian", "Normal" },
			{ "Mantle of the Forlorn Vanquisher", "Cho'gall", "Normal" }
		};
		PPW = { --Paladin, Priest, Warlock
			{ "Crown of the Forlorn Conqueror", "Nefarian or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Shoulders of the Forlorn Conqueror", "Cho'gall or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Chest of the Forlorn Conqueror", "Halfus Wyrmbreaker or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Gauntlets of the Forlorn Conqueror", "Magmaw or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Leggings of the Forlorn Conqueror", "Maloriak or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Helm of the Forlorn Conqueror", "Nefarian", "Normal" },
			{ "Mantle of the Forlorn Conqueror", "Cho'gall", "Normal" }
		};
		HSW = { --Hunter, Shaman, Warrior
			{ "Crown of the Forlorn Protector", "Nefarian or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Shoulders of the Forlorn Protector", "Cho'gall or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Chest of the Forlorn Protector", "Halfus Wyrmbreaker or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Gauntlets of the Forlorn Protector", "Magmaw or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Leggings of the Forlorn Protector", "Maloriak or \"Essence of the Forlorn\" token", "Heroic" },
			{ "Helm of the Forlorn Protector", "Nefarian", "Normal" },
			{ "Mantle of the Forlorn Protector", "Cho'gall", "Normal" }
		};
	};
};
----------------------------------------------INSTANCES---------------------------------------------
local Raids = {
	"Baradin Hold",
	"Bastion of Twilight",
	"Blackwing Descent",
	"Throne of the Four Winds"
};

local HeroicDungeons = { --Heroic dungeons only
	"Zul'Aman",
	"Zul'Gurub",
	"The Deadmines",
	"Shadowfang Keep"
};

local Dungeons = { --Normal & Heroic dungeons
	"Throne of the Tides",
	"Blackrock Caverns",
	"The Stonecore",
	"The Vortex Pinnacle",
	"Lost City of Tol'vir",
	"Grim Batol",
	"Halls of Origination"
};
-----------------------------------------------BOSSES-----------------------------------------------
local BaradinHoldBosses = {
	"Argaloth",
	"(Trash Drop)"
};

local BasitonOfTwilightBosses = {
	"Halfus Wyrmbreaker",
	"Valiona & Theralion",
	"Ascendent Council",
	"Cho'gall",
	"Sinestra",
	"(Trash Drop)"
};

local BlackwingDescentBosses = {
	"Omnotron Defense System",
	"Magmaw",
	"Maloriak",
	"Atramedes",
	"Chimaeron",
	"Nefarian",
	"(Trash Drop)"
};

local ThroneOfTheFourWindsBosses = {
	"Conclave of Wind",
	"Al'Akir"
};

local ZulAmanBosses = {
	"Nalorakk",
	"Akil'zon",
	"Jan'alai",
	"Halazzi",
	"Hex Lord Malacrass",
	"Daakara",
	"(Timed Event)",
	"(Trash Drop)"
};

local ZulGurubBosses = {
	"High Priest Venoxis",
	"Bloodlord Mandokir",
	"High Priestess Kilnara",
	"Zanzil",
	"Jin'do the Godbreaker",
	"Gri'lek",
	"Hazza'rah",
	"Renataki",
	"Wushoolay",
	"(Trash Drop)"
};

local ThroneOfTidesBosses = {
	"Lady Naz'jar",
	"Commander Ulthok",
	"Mindbender Ghur'sha",
	"Ozumat",
	"(Trash Drop)"
};

local BlackrockCavernsBosses = {
	"Rom-ogg Bonecrusher",
	"Corla, Herald of Twilight",
	"Karsh Steelbender",
	"Beauty",
	"Ascendant Lord Obsidius",
	"(Trash Drop)"
};

local StonecoreBosses = {
	"Corborus",
	"Slabhide",
	"Ozruk",
	"High Priestess Azil",
	"(Trash Drop)"
};

local VortexPinnacleBosses = {
	"Grand Vizier Ertan",
	"Altairus",
	"Asaad",
	"(Trash Drop)"
};

local LostCityOfTolvirBosses = {
	"General Husam",
	"Augh and Lockmaw",
	"High Prophet Barim",
	"Siamat",
	"(Trash Drop)"
};

local GrimBatolBosses = {
	"General Umbriss",
	"Forgemaster Throngus",
	"Drahga Shadowburner",
	"Erudax",
	"(Trash Drop)"
};

local HallsOfOriginationBosses = {
	"Temple Guardian Anhuur",
	"Earthrager Ptah",
	"Anraphet",
	"Isiset",
	"Ammunae",
	"Setesh",
	"Rajh",
	"(Trash Drop)"
};

local DeadminesBosses = {
	"Glubtok",
	"Helix Gearbreaker",
	"Foe Reaper 5000",
	"Admiral Ripsnarl",
	"\"Captain\" Cookie",
	"Vanessa VanCleef",
	"(Trash Drop)"
};

local ShadowfangKeepBosses = {
	"Baron Ashbury",
	"Baron Silverlaine",
	"Commander Springvale",
	"Lord Walden",
	"Lord Godfrey",
	"(Trash Drop)"
};
--------------------------------------------CONSTRUCTORS--------------------------------------------
function UpgradeList:TopLevel(info, level)
	for i = 1, #GeneralItemLocations, 1 do
		info = UIDropDownMenu_CreateInfo()
		info.text = GeneralItemLocations[i]
		info.notCheckable = 1
		if info.text == "Other" then
			function info.func()
				UIDropDownMenu_SetText(DropDown_Button, "Other");
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = "Other";
				CloseDropDownMenus(1);
			end
			UIDropDownMenu_AddButton(info, level)
		else
			info.hasArrow = true
			info.value = info.text
			UIDropDownMenu_AddButton(info, level)
		end
	end
end

function UpgradeList:RaidSubmenu(info, level)
	for i = 1, #InstanceTypes, 1 do
		info.text = InstanceTypes[i]
		info.hasArrow = true
		info.value = InstanceTypes[i] .. "Raid"
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:DungeonSubmenu(info, level)
	for i = 1, #InstanceTypes, 1 do
		info.text = InstanceTypes[i]
		info.hasArrow = true
		info.value = InstanceTypes[i] .. "Dungeon"
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:CraftedSubmenu(info, level)
	for i = 1, #Professions, 1 do
		info.text = Professions[i]
		info.value = info.text
		info.notCheckable = true
		function info.func()
			UIDropDownMenu_SetText(DropDown_Button, Professions[i]);
			UpgradeListDBVar.char.SavedOptions.SelectedInstance = Professions[i];
			CloseDropDownMenus(1);
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:Reputations(info, level)
	for i = 1, #Reputations, 1 do
		info.text = Reputations[i]
		info.hasArrow = true
		info.value = info.text
		info.notCheckable = true
		UIDropDownMenu_AddButton(info, level)
		local rep = info.value
		UpgradeList:RepLevels(info, level, rep)
	end
	if UnitFactionGroup("player") == "Alliance" then
		for i = 1, #AllianceReputations, 1 do
			info.text = AllianceReputations[i]
			info.hasArrow = true
			info.value = info.text
			info.notCheckable = true
			UIDropDownMenu_AddButton(info, level)
		end
	elseif UnitFactionGroup("player") == "Horde" then
		for i = 1, #HordeReputations, 1 do
			info.text = HordeReputations[i]
			info.hasArrow = true
			info.value = info.text
			UIDropDownMenu_AddButton(info, level)
		end
	else
		print("UpgradeList: Unable to determine your faction!")
	end
	if IsInGuild() then
		info.text = GetGuildInfo("player")
		info.hasArrow = true
		info.value = info.text
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:RepLevels(info, level, rep)
	level = level + 1
	if level == 3 then
		local info = UIDropDownMenu_CreateInfo()
		for i = 1, #ReputationLevels, 1 do
			info.text = ReputationLevels[i]
			info.value = ReputationLevels[i]
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(DropDown_Button, rep)
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = ReputationLevels[i]
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = rep
				CloseDropDownMenus(1)
			end
			--print(level)
			--print(info.text)
			--print(info.value)
			--print(rep)
			--UIDropDownMenu_AddButton(info, level)
		end
	end
end

function UpgradeList:Purchasable(info, level)
	for i = 1, #Currency, 1 do
		info.text = Currency[i]
		info.value = info.text
		info.notCheckable = 1
		if Currency[i] == "Gear Token" then
			info.hasArrow = true
		else
			info.hasArrow = false
			function info.func()
				CloseDropDownMenus(1);
				UIDropDownMenu_SetSelectedName(DropDown_Button, Currency[i]);
				PointInput:Show();
				PointInputBox:SetFocus();
				UpgradeListMainFrame:EnableMouse(false)
				CloseButton:EnableMouse(false)
				ToggleButton:EnableMouse(false)
				UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(false)
				UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(false)
				OptionsFrame:EnableMouse(false)
				ItemEntryBox:EnableMouse(false)
				DropDown_Button:EnableMouse(false)
				NoteEntryBox:EnableMouse(false)
				PrimarySpecRadioButton:EnableMouse(false)
				SecondarySpecRadioButton:EnableMouse(false)
				priorityfield:EnableMouse(false)
				AddButton:EnableMouse(false)
				tofield:EnableMouse(false)
				fromfield:EnableMouse(false)
				MoveButton:EnableMouse(false)
				DeleteButton:EnableMouse(false)
				deletefield:EnableMouse(false)
			end
		end
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:HeroicRaids(info, level)
	for i = 1, #Raids, 1 do
		info.text = Raids[i]
		info.value = Raids[i] .. " Heroic"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:NormalRaids(info, level)
	for i = 1, #Raids, 1 do
		info.text = Raids[i]
		info.value = Raids[i] .. " Normal"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:HeroicDungeons(info, level)
	for i = 1, #HeroicDungeons, 1 do
		info.text = HeroicDungeons[i]
		info.value = HeroicDungeons[i] .. " Heroic"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
	for i = 1, #Dungeons, 1 do
		info.text = Dungeons[i]
		info.value = Dungeons[i] .. " Heroic"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end

function UpgradeList:NormalDungeons(info, level)
	for i = 1, #Dungeons, 1 do
		info.text = Dungeons[i]
		info.value = Dungeons[i] .. " Normal"
		info.hasArrow = true
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	end
end